import requests
import json
from .tromero_utils import mock_openai_format_stream
from .constants import DATA_URL, BASE_URL


def post_data(data, auth_token):
    headers = {
        'X-API-KEY': auth_token,
        'Content-Type': 'application/json'
    }
    try:
        response = requests.post(DATA_URL, json=data, headers=headers)
        response.raise_for_status()  # Raises HTTPError for bad responses (4XX, 5XX)
        return response.json()  # Return the JSON response if request was successful
    except Exception as e:
        return {'error': f'An error occurred: {e}', 'status_code': response.status_code if 'response' in locals() else 'N/A'}
    

def tromero_model_create(model, model_url, messages, tromero_key, parameters={}):
    headers = {'Content-Type': 'application/json'}
    data = {
        "adapter_name": model,
        "messages": messages,
        "parameters": parameters
    }
    headers['X-API-KEY'] = tromero_key
    try:
        response = requests.post(f"{model_url}/generate", json=data, headers=headers)
        response.raise_for_status()  # Raises HTTPError for bad responses (4XX, 5XX)
        return response.json()  # Return the JSON response if request was successful
    except Exception as e:
        return {'error': f'An error occurred: {e}', 'status_code': response.status_code if 'response' in locals() else 'N/A'}
    

def get_model_url(model_name, auth_token):
    headers = {
        'X-API-KEY': auth_token,
        'Content-Type': 'application/json'
    }
    try:
        response = requests.get(f"{BASE_URL}/model/{model_name}/url", headers=headers)
        response.raise_for_status()  # Raises HTTPError for bad responses (4XX, 5XX)
        return response.json()['url'], response.json().get('base_model', False)  # Return the JSON response if request was successful
    except Exception as e:
        print(f"error: {e}")
        return {'error': f'An error occurred: {e}', 'status_code': response.status_code if 'response' in locals() else 'N/A'}
    
class StreamResponse:
    def __init__(self, response):
        self.response = response

    def __iter__(self):
        try:
            last_chunk = None
            for chunk in self.response.iter_content(chunk_size=10000000):
                # chunk_dict = json.loads(chunk)
                chunk = chunk.decode('utf-8')
                json_str = chunk[5:]
                last_chunk = json_str
                chunk_dict = json.loads(json_str)
                formatted_chunk = mock_openai_format_stream(chunk_dict['token']['text'])
                yield formatted_chunk
        except Exception as e:
            print(f"Error: {e}")
            return
    
def tromero_model_create_stream(model, model_url, messages, tromero_key, parameters={}):
    headers = {'Content-Type': 'application/json'}
    data = {
        "adapter_name": model,
        "messages": messages,
        "parameters": parameters
    }
    headers['X-API-KEY'] = tromero_key
    try:
        response = requests.post(model_url + "/generate_stream", json=data, headers=headers, stream=True)
        return StreamResponse(response), None
    except Exception as e:
        return None, {'error': f'An error occurred: {e}', 'status_code': response.status_code if 'response' in locals() else 'N/A'}



    




import requests
import json
from .constants import DATA_URL, BASE_URL

def genric_request(method, path, data, tromero_key):
    headers = {'Content-Type': 'application/json',
               'X-API-KEY': tromero_key}
    if method == "GET":
        response = requests.get(f"{BASE_URL}{path}", headers=headers)
    else :
        response = requests.request(method, f"{BASE_URL}{path}", json=data, headers=headers)
    response.raise_for_status()  # Raises HTTPError for bad responses (4XX, 5XX)
    return response.json()  # Return the JSON response if request was successful


def get_signed_url(auth_token):
    headers = {
        'X-API-KEY': auth_token,
        'Content-Type': 'application/json'
    }
    response = requests.get(f"{BASE_URL}/generate_signed_url", headers=headers)
    response.raise_for_status()  # Raises HTTPError for bad responses (4XX, 5XX)
    return response.json()['signedUrl'], response.json()['filename']  # Return the JSON response if request was successful
    

def upload_file_to_url(signed_url, file_path):
    # Open the file in binary mode
    with open(file_path, 'rb') as f:
        headers = {'Content-Type': 'application/octet-stream'}  # Adjust this based on the actual file type you are uploading.
        response = requests.put(signed_url, data=f, headers=headers)

    # Check if the upload was successful
    if response.status_code != 200:
        raise Exception(f"An error occurred in upload: {response.text}")
    

def save_logs(custom_logs_filename, save_logs_with_tags, tromero_key, make_synthetic_version=False):
    headers = {'Content-Type': 'application/json'}
    data = {
        "custom_logs_filename": custom_logs_filename,
        "save_logs_with_tags": save_logs_with_tags,
        "make_synthetic_version": bool(make_synthetic_version)
    }
    headers['X-API-KEY'] = tromero_key
    response = requests.post(f"{BASE_URL}/custom_log_upload", json=data, headers=headers)
    response.raise_for_status()  # Raises HTTPError for bad responses (4XX, 5XX)
    return response.json()  # Return the JSON response if request was successful
    
def create_fine_tuning_job(data, tromero_key):
    headers = {'Content-Type': 'application/json'}
    headers['X-API-KEY'] = tromero_key
    
    response = requests.post(f"{BASE_URL}/training-pod", json=data, headers=headers)
    return response.json()  # Return the JSON response if request was successful

def get_models(tromero_key):
    headers = {
        'X-API-KEY': tromero_key,
        'Content-Type': 'application/json'
    }
    response = requests.get(f"{BASE_URL}/models?show_full=true", headers=headers)
    response.raise_for_status()  # Raises HTTPError for bad responses (4XX, 5XX)
    return response.json()  # Return the JSON response if request was successful

def get_model_training_info(model_name, tromero_key):
    headers = {
        'X-API-KEY': tromero_key,
        'Content-Type': 'application/json'
    }
    response = requests.get(f"{BASE_URL}/named-training-info-log/{model_name}", headers=headers)
    response.raise_for_status()  # Raises HTTPError for bad responses (4XX, 5XX)
    return response.json()  # Return the JSON response if request was successful

def deploy_model_request(model_name, tromero_key):
    headers = {
        'X-API-KEY': tromero_key,
        'Content-Type': 'application/json'
    }

    data = {
        "model_name": model_name
    }
    response = requests.post(f"{BASE_URL}/deploy_model", json=data, headers=headers)
    response.raise_for_status()  # Raises HTTPError for bad responses (4XX, 5XX)
    return response.json()  # Return the JSON response if request was successful

def get_model_request(model_name, tromero_key):
    headers = {
        'X-API-KEY': tromero_key,
        'Content-Type': 'application/json'
    }
    response = requests.get(f"{BASE_URL}/models/by_name/{model_name}", headers=headers)
    response.raise_for_status()  # Raises HTTPError for bad responses (4XX, 5XX)
    return response.json()  # Return the JSON response if request was successful
    
def undeploy_model_request(model_name, tromero_key):
    headers = {
        'X-API-KEY': tromero_key,
        'Content-Type': 'application/json'
    }

    data = {
        "model_name": model_name
    }
    response = requests.post(f"{BASE_URL}/undeploy_model", json=data, headers=headers)
    response.raise_for_status()  # Raises HTTPError for bad responses (4XX, 5XX)
    return response.json()  # Return the JSON response if request was successful

def get_tags(tromero_key):
    headers = {
        'X-API-KEY': tromero_key,
    }
    response = requests.get(f"{BASE_URL}/tags", headers=headers)
    response.raise_for_status()  # Raises HTTPError for bad responses (4XX, 5XX)
    return response.json()  # Return the JSON response if request was successful

def create_dataset(name, description, tags, tromero_key):
    data = {
        "name": name,
        "description": description,
        "tags": tags
    }
    headers = {'Content-Type': 'application/json'}
    headers['X-API-KEY'] = tromero_key
    response = requests.post(f"{BASE_URL}/datasets", json=data, headers=headers)
    response.raise_for_status()  # Raises HTTPError for bad responses (4XX, 5XX)
    return response.json()  # Return the JSON response if request was successful

def model_evaluation_request(model_name, tromero_key):
    path = f"/evaluate/named/{model_name}"
    return genric_request("GET", path, {}, tromero_key)
    


import unittest
from unittest.mock import patch, MagicMock
import datetime
from tromero_tailor import TailorAI
from tromero_tailor.wrapper import MockCompletions
from unittest.mock import patch, MagicMock, mock_open

from pydantic import BaseModel
from typing import List, Literal, Optional

class Message(BaseModel):
    content: str
    role: Literal["assistant", "user", "system"]

class Choice(BaseModel):
    finish_reason: str
    index: int
    logprobs: Optional[dict] = None
    message: Message

class CompletionUsage(BaseModel):
    """Placeholder for usage stats, define as needed"""
    tokens: int

class ChatCompletion(BaseModel):
    id: str
    choices: List[Choice]
    created: int
    model: str
    object: Literal["chat.completion"]
    system_fingerprint: Optional[str] = None
    usage: Optional[CompletionUsage] = None

class TestModel:
    def __init__(self, id):
        self.id = id
        
import unittest
from unittest.mock import patch
from tromero_tailor import TailorAI

# Assuming the classes are defined as you provided above

def create_chat_completion():
    return ChatCompletion(
        id="1234abcd",
        choices=[
            Choice(
                finish_reason='complete',
                index=0,
                message=Message(content="Test response", role="assistant"),
                logprobs=None
            )
        ],
        created=1701070800,
        model="gpt-4",
        object="chat.completion",
        system_fingerprint="example_fingerprint",
        usage=CompletionUsage(tokens=100)
    )

class TestCreateChatCompletion(unittest.TestCase):
    @patch('openai.resources.chat.completions.Completions.create', autospec=True) 
    @patch('openai.resources.models.Models.list', autospec=True)
    @patch('tromero_tailor.wrapper.post_data', autospec=True)
    def test_create_with_openai_model(self, mock_post, mock_list_models, mock_create):
        # Setup the mocked return value for the create method
        mock_create.return_value = create_chat_completion()
        mock_list_models.return_value = [TestModel("valid_model")]
        mock_post.return_value = None

        client = TailorAI(api_key="fake_api_key", tromero_key="fake_tromero_key")
        result = client.chat.completions.create(model="valid_model", messages=["Hello, world!"])
        
        # Assertions to verify behavior
        self.assertIsNotNone(result.choices)
        self.assertEqual(result.choices[0].message.content, "Test response")
        # check post_data is called
        mock_post.assert_called()

    @patch('openai.resources.chat.completions.Completions.create', autospec=True)
    @patch('openai.resources.models.Models.list', autospec=True)
    @patch('tromero_tailor.wrapper.post_data', autospec=True)
    def test_create_with_no_choices_returned(self, mock_post, mock_list_models, mock_create):
        mock_create.return_value = ChatCompletion(
            id="5678efgh",
            choices=[],
            created=1701070800,
            model="gpt-4",
            object="chat.completion",
            system_fingerprint="example_fingerprint",
            usage=CompletionUsage(tokens=50)
        )
        mock_list_models.return_value = [TestModel("valid_model")]
        mock_post.return_value = None

        client = TailorAI(api_key="fake_api_key", tromero_key="fake_tromero_key")
        result = client.chat.completions.create(model="valid_model", messages=["Hello, world!"])
        
        self.assertIsInstance(result, ChatCompletion)
        self.assertEqual(len(result.choices), 0)
        mock_post.assert_not_called()

# Run the test
if __name__ == '__main__':
    unittest.main()






import json
from openai import OpenAI
from openai.resources import Chat
from openai.resources.chat.completions import (
    Completions
)
from openai._compat import cached_property
import datetime
from tromero.tromero_requests import post_data, tromero_model_create, get_model_url, tromero_model_create_stream
from tromero.tromero_utils import mock_openai_format, tags_to_string
import warnings
import threading
from tromero.fine_tuning import TromeroModels, TromeroData, FineTuningJob, Datasets
from jsonschema import Draft7Validator
from jsonschema.exceptions import SchemaError
# import check_schema
from jsonschema import Validator, FormatChecker
from jsonschema import Draft7Validator


class MockCompletions(Completions):
    def __init__(self, client):
        super().__init__(client)

    def _choice_to_dict(self, choice):
        return {
            "message": {
                "content": choice.message.content,
                "role": choice.message.role,
            }
        }
    
    def _save_data(self, data):
        if self._client.save_data:
            threading.Thread(target=post_data, args=(data, self._client.tromero_key)).start()

    def validate_schema(self, schema):
        try:
        # Validate schema against the JSON Schema Draft 7
            Draft7Validator.check_schema(schema)

            # Additional common checks
            if "properties" in schema:
                for prop, details in schema["properties"].items():
                    if "type" in details:
                        valid_types = {"string", "number", "integer", "boolean", "array", "object"}
                        if details["type"] not in valid_types:
                            raise ValueError(f"Invalid type specified: {details['type']} in property '{prop}'")
                    else:
                        raise ValueError(f"No type specified for property '{prop}'")
            else:
                raise ValueError("No properties defined in the schema.")

            print("Detailed validation passed.")
        except (SchemaError, ValueError) as e:
            raise SchemaError(f"Schema validation failed: {e}")


    def _format_kwargs(self, kwargs):
        keys_to_keep = [
            "best_of", "decoder_input_details", "details", "do_sample", 
            "max_new_tokens", "ignore_eos_token", "repetition_penalty", 
            "return_full_outcome", "seed", "stop", "temperature", "top_k", 
            "top_p", "truncate", "typical_p", "watermark", 
            "adapter_id", "adapter_source", "merged_adapters", "response_format", 
            "make_synthetic_version", "guided_schema", "guided_regex", "tools"
        ]
        invalid_key_found = False
        parameters = {}
        for key in kwargs:
            if key not in keys_to_keep and key not in ["tags", "model", "messages", "use_fallback", "fallback_model", "stream"]:
                warnings.warn(f"Warning: {key} is not a valid parameter for the model. This parameter will be ignored.")
                invalid_key_found = True
            elif key in keys_to_keep:
                parameters[key] = kwargs[key]

        if invalid_key_found:
            print("the following parameters are valid for the model: ", keys_to_keep)
        
        return parameters


    def _format_messages(self, messages):
        system_prompt = ""
        num_prompts = 0
        for message in messages:
            if message['role'] == "system":
                system_prompt += message['content'] + " "
                num_prompts += 1
            else:
                break
        if num_prompts <= 1:
            return messages

        messages = [{"role": "system", "content": system_prompt}] + messages[num_prompts:]
        print("Warning: Multiple system prompts will be combined into one prompt when saving data or calling custom models.")
        return messages
    
    def _tags_to_string(self, tags):
        return ",".join(tags)
    
    def _stream_response(self, response, init_save_data, fall_back_dict):
        try:
            full_message = ''
            for chunk in response:
                if chunk:
                    if chunk.choices[0].delta.content and chunk.choices[0].delta.content != '</s>':
                        full_message += str(chunk.choices[0].delta.content)
                    yield chunk
        except Exception as e:
            print("Error streaming response:", e, flush=True)
            raise e
        finally:
            if init_save_data != {}:
                init_save_data['messages'].append({"role": "assistant", "content": full_message})
                self._save_data(init_save_data)


    def check_model(self, model):
        try:
            models = self._client.models.list()
        except:
            return False
        model_names = [m.id for m in models]
        return model in model_names
    
    def create(self, *args, **kwargs):
        messages = kwargs['messages']
        formatted_messages =  self._format_messages(messages)
        model = kwargs['model']
        stream = kwargs.get('stream', False)
        tags = kwargs.get('tags', [])
        send_kwargs = {}
        use_fallback = kwargs.get('use_fallback', True)
        fallback_model = kwargs.get('fallback_model', '')
        
        openai_kwargs = {k: v for k, v in kwargs.items() if k not in ['tags', 'use_fallback', 'fallback_model']}
        if self.check_model(kwargs['model']):
            res = Completions.create(self, *args, **openai_kwargs)  
            send_kwargs = openai_kwargs
        else:
            formatted_kwargs = self._format_kwargs(kwargs)
            send_kwargs = formatted_kwargs
            model_name = model
            if model_name not in self._client.model_urls:
                url, base_model = get_model_url(model_name, self._client.tromero_key)
                self._client.model_urls[model_name] = url
                self._client.is_base_model[model_name] = base_model
            model_request_name = model_name if not self._client.is_base_model[model_name] else "NO_ADAPTER"
            if stream:
                res, e =  tromero_model_create_stream(model_request_name, self._client.model_urls[model_name], formatted_messages, self._client.tromero_key, parameters=formatted_kwargs)
                if e:
                    if use_fallback and fallback_model:
                        print("Error in making request to model. Using fallback model.")
                        kwargs['model'] = fallback_model
                        kwargs['use_fallback'] = False
                        return self.create(*args, **kwargs)

            else:
                res = tromero_model_create(model_request_name, self._client.model_urls[model_name], formatted_messages, self._client.tromero_key, parameters=formatted_kwargs)
                # check if res has field 'generated_text'
                if 'generated_text' in res:
                    generated_text = res['generated_text']
                    usage = res['usage']
                    res = mock_openai_format(generated_text, usage)

        if hasattr(res, 'choices'):
            for choice in res.choices:
                formatted_choice = self._choice_to_dict(choice)
                save_data = {"messages": formatted_messages + [formatted_choice['message']],
                                "model": model,
                                "kwargs": send_kwargs,
                                "creation_time": str(datetime.datetime.now().isoformat()),
                                "tags": tags_to_string(tags)
                                }
                # if hasattr(res, 'usage'):
                #     save_data['usage'] = res.usage.model_dump()
                self._save_data(save_data)
        elif stream:
            init_save_data = {"messages": formatted_messages,
                                "model": model,
                                "kwargs": send_kwargs,
                                "creation_time": str(datetime.datetime.now().isoformat()),
                                "tags": tags_to_string(tags)
                                }
            fall_back_dict = {}
            if use_fallback and fallback_model:
                kwargs['model'] = fallback_model
                kwargs['use_fallback'] = False
                fall_back_dict = {
                    'args': args,
                    'kwargs': kwargs
                }
            return self._stream_response(res, init_save_data, fall_back_dict)
        else:
            if use_fallback and fallback_model:
                print("Error in making request to model. Using fallback model.")
                kwargs['model'] = fallback_model
                kwargs['use_fallback'] = False
                return self.create(*args, **kwargs)

        return res


class MockChat(Chat):
    def __init__(self, client):
        super().__init__(client)

    @cached_property
    def completions(self) -> Completions:
        return MockCompletions(self._client)


class Tromero(OpenAI):
    chat: MockChat
    def __init__(self, tromero_key, api_key="", save_data=False):
        super().__init__(api_key=api_key)
        self.current_prompt = []
        self.model_urls = {}
        self.is_base_model = {}
        self.tromero_key = tromero_key
        self.chat = MockChat(self)
        self.save_data = save_data
        self.tromero_models = TromeroModels(tromero_key)
        self.fine_tuning_jobs = FineTuningJob(tromero_key)
        self.data = TromeroData(tromero_key)
        self.datasets = Datasets(tromero_key)



from .fine_tuning_requests import (get_signed_url, upload_file_to_url, save_logs, create_fine_tuning_job,
                                   get_model_training_info, get_models, deploy_model_request, get_model_request, undeploy_model_request, 
                                   get_tags, create_dataset, model_evaluation_request)
from .tromero_utils import tags_to_string, validate_file_content
from .fine_tuning_models import Model, TrainingMetrics, Dataset
import uuid
import json

def set_raw(val, default):
    return val if val is not None else default
    
class Datasets:
    def __init__(self, tromero_key, raw_default=False):
        self.tromero_key = tromero_key
        self.raw_default = raw_default

    def create_from_file(self, file_path, name, description, tags):
        id_tag = f"dataset_tag_{str(uuid.uuid4())}"
        if type(tags) == str:
            tags = [tags]
        tags = list(tags)
        tags.append(id_tag)
        if not validate_file_content(file_path):
            return
        signed_url, filename = get_signed_url(self.tromero_key)
        upload_file_to_url(signed_url, file_path)
        save_logs(filename, tags, self.tromero_key)
        print(f"File uploaded successfully! Tags: {tags}")
        create_dataset(name, description, [id_tag], self.tromero_key)
        return True
    
    def create_from_tags(self, name, description, tags):
        create_dataset(name, description, tags, self.tromero_key)
        return True
    
    def list(self, raw=None):
        raw = set_raw(raw, self.raw_default)
        response = get_tags(self.tromero_key)
        datasets = response.get("datasets", [])
        if raw:
            return datasets
        return [Dataset(**dataset) for dataset in datasets]
    
class FineTuningJob:
    def __init__(self, tromero_key, raw_default=False):
        self.tromero_key = tromero_key
        self.raw_default = raw_default

    # Valid parameters for fine tuning
    # epoch, learning_rate, batch_size, tags, custom_logs_filename, save_logs_with_tags, custom_dataset, skip_logs_with_errors
    def create(self, model_name, base_model, parameters=None):
        data = {"model_name": model_name, "base_model": base_model}   
        if parameters:
            if type(parameters) == str:
                parameters = json.loads(parameters)
                print(parameters)
            data.update(parameters)
        response = create_fine_tuning_job(data, self.tromero_key)
        return response
    
    def get_metrics(self, model_name, raw=None):
        raw = set_raw(raw, self.raw_default)
        response = get_model_training_info(model_name, self.tromero_key)
        metrics = response.get("metrics", {})
        if not metrics:
            print("Metrics are not available for this model yet.")
        if raw:
            return metrics
        ret = TrainingMetrics(**metrics)
        return ret

    
class TromeroModels:
    def __init__(self, tromero_key, raw_default=False):
        self.tromero_key = tromero_key
        self.raw_default = raw_default

    def list(self, raw=None):
        """Returns a list of the users fine tuned models"""
        raw = set_raw(raw, self.raw_default)
        response = get_models(self.tromero_key)
        if raw:
            return response["message"]
        model_data = [Model(**model) for model in response["message"]]
        return model_data
    
    def deploy(self, model_name):
        """Deploys a fine tuned model. Model must be undeoloyed to work. Takes model_name"""
        response = deploy_model_request(model_name, self.tromero_key)
        return response
    
    def get_info(self, model_name, raw=None):
        """Returns information about the model. Takes model_name"""
        raw = set_raw(raw, self.raw_default)
        response = get_model_request(model_name, self.tromero_key)
        if raw:
            return response
        model_data = Model(**response["message"])
        return model_data
    
    def undeploy(self, model_name):
        """Undeploys a fine tuned model. Model must be deployed to work. Takes model_name"""
        response = undeploy_model_request(model_name, self.tromero_key)
        return response

    
class TromeroData:
    def __init__(self, tromero_key):
        self.tromero_key = tromero_key

    def upload(self, file_path, tags, make_synthetic_version=False):
        if type(tags) == str:
            tags = [tags]
        tags = list(tags)
        if not validate_file_content(file_path):
            return
        signed_url, filename = get_signed_url(self.tromero_key)
        upload_file_to_url(signed_url, file_path)
        save_logs(filename, tags, self.tromero_key, make_synthetic_version)
        print(f"File uploaded successfully! Tags: {tags}")
        return True
    
    def get_tags(self):
        response = get_tags(self.tromero_key)
        return response["message"]
    

# class Evaluations:
#     def __init__(self, tromero_key, raw_default=False):
#         self.tromero_key = tromero_key
#         self.raw_default = raw_default

#     def model_evaluation(self, model_name, raw=None):
#         raw = set_raw(raw, self.raw_default)
#         response = model_evaluation_request(model_name, self.tromero_key)
#         return response
#         if raw:
#             return response
#         evaluation = response.get("evaluation", {})
#         return evaluation

        


import json

class ModelConfig:
    def __init__(self, base_model, batch_size=None, epoch=None, learning_rate=None, custom_dataset=None, custom_logs_filename=None, 
                 number_of_bad_logs=None, save_logs_with_tags=None, tags=None, external_service_id=None, pod_id=None):
        self.base_model = base_model
        self.batch_size = batch_size
        self.epoch = epoch
        self.learning_rate = learning_rate
        self.custom_dataset = custom_dataset
        self.custom_logs_filename = custom_logs_filename
        self.number_of_bad_logs = number_of_bad_logs
        self.save_logs_with_tags = save_logs_with_tags
        self.tags = tags
        self.external_service_id = external_service_id
        self.pod_id = pod_id

class UsageData:
    def __init__(self, date, tokens):
        self.date = date
        self.tokens = tokens


class BaseModelData:
    def __init__(self, available_for_finetuning, available_for_inference, default_batch_size, default_lr, display_name, hf_repo, id, model_name,
                 model_size, supported_context_len, training_time_per_log, training_time_y_intercept):
        self.available_for_finetuning = available_for_finetuning
        self.available_for_inference = available_for_inference
        self.default_batch_size = default_batch_size
        self.default_lr = default_lr
        self.display_name = display_name
        self.hf_repo = hf_repo
        self.id = id
        self.model_name = model_name
        self.model_size = model_size
        self.supported_context_len = supported_context_len
        self.training_time_per_log = training_time_per_log
        self.training_time_y_intercept = training_time_y_intercept

class EvaluationState:
    def __init__(self, status):
        self.status = status


class ModelEvaluationState:
    def __init__(self, mix_eval, needlehaystack):
        self.mix_eval = EvaluationState(**mix_eval)
        self.needlehaystack = EvaluationState(**needlehaystack)


class Model:
    def __init__(self, model_id, model_name, state, model_config, cost_per_1000_tokens=0, created_at=None, created_at_unix=None, 
                 last_deployed_on=None, last_deployed_on_unix=None, last_used=None, last_used_unix=None, training_ended_at=None, 
                 training_ended_at_unix=None, updated_at=None, user_id=None, model_evaluation=None, self_hosted=False, server_id=None,
                 usage_data=[], base_model_data=None, base_model_id=None, model_evaluation_state=None):
        self.model_id = model_id
        self.model_name = model_name
        self.state = state
        self.cost_per_1000_tokens = cost_per_1000_tokens
        self.created_at = created_at
        self.created_at_unix = created_at_unix
        self.last_deployed_on = last_deployed_on
        self.last_deployed_on_unix = last_deployed_on_unix
        self.last_used = last_used
        self.last_used_unix = last_used_unix
        self.training_ended_at = training_ended_at
        self.training_ended_at_unix = training_ended_at_unix
        self.updated_at = updated_at
        self.user_id = user_id
        self.model_evaluation = model_evaluation  
        self.self_hosted = self_hosted
        self.server_id = server_id
        self.usage = [UsageData(**data) for data in usage_data]
        self.model_config = ModelConfig(**model_config)
        self.base_model_data = BaseModelData(**base_model_data)
        self.base_model_id = base_model_id
        self.model_evaluation_state = ModelEvaluationState(**model_evaluation_state)

class TrainingMetrics:
    def __init__(self, eval_loss, loss, eval_perplexity, perplexity):
        self.eval_loss = eval_loss
        self.loss = loss
        self.eval_perplexity = eval_perplexity
        self.perplexity = perplexity


class Filter:
    def __init__(self, from_date, models, tags, to_date):
        self.from_date = from_date
        self.models = models
        self.tags = tags
        self.to_date = to_date

class Dataset:
    def __init__(self, id, name, description, filters, user_id, created_at=None, updated_at=None):
        self.dataset_id = id
        self.name = name
        self.description = description
        self.filters = Filter(**filters)
        self.user_id = user_id
        self.created_at = created_at
        self.updated_at = updated_at

# {
#   "evaluation": {
#     "mix_eval": {
#       "global_status": "partially_complete",
#       "ongoing_tests": [
#         "GSM8K",
#         "AGIEVAL",
#         "TRIVIAQA",
#         "HELLASWAG",
#         "BOOLQ",
#         "OPENBOOKQA",
#         "COMMONSENSEQA"
#       ],
#       "results": {
#         "ARC": 0.859,
#         "BBH": 0.755,
#         "DROP": 0.746,
#         "GPQA": 0.5,
#         "MATH": 0.57,
#         "MBPP": 0.0,
#         "MMLU": 0.662,
#         "PIQA": 0.823,
#         "SIQA": 0.684,
#         "overall score (final score)": 0.6221111111111111
#       },
#       "status": {
#         "AGIEVAL": "started",
#         "ARC": "complete",
#         "BBH": "complete",
#         "BOOLQ": "started",
#         "COMMONSENSEQA": "started",
#         "DROP": "complete",
#         "GPQA": "complete",
#         "GSM8K": "started",
#         "HELLASWAG": "started",
#         "MATH": "complete",
#         "MBPP": "complete",
#         "MMLU": "complete",
#         "OPENBOOKQA": "started",
#         "PIQA": "complete",
#         "SIQA": "complete",
#         "TRIVIAQA": "started"
#       }
#     },
#     "needlehaystack": {
#       "global_status": "complete",
#       "results": {
#         "scores": [
#           [
#             10,
#             10,
#             10,
#             10,
#             10
#           ],
#           [
#             10,
#             10,
#             10,
#             10,
#             10
#           ],
#           [
#             10,
#             10,
#             10,
#             10,
#             10
#           ],
#           [
#             10,
#             10,
#             10,
#             10,
#             10
#           ],
#           [
#             3,
#             1,
#             3,
#             7,
#             10
#           ],
#           [
#             10,
#             3,
#             1,
#             3,
#             10
#           ]
#         ],
#         "x_axis": [
#           1000,
#           2400,
#           3800,
#           5200,
#           6600,
#           8000
#         ],
#         "y_axis": [
#           0,
#           25,
#           50,
#           75,
#           100
#         ]
#       },
#       "status": "complete"
#     }
#   },
#   "status": "success"
# }


# class MixEval:
#     def __init__(self, global_status, ongoing_tests, results, status):
#         self.global_status = global_status
#         self.ongoing_tests = ongoing_tests
#         self.results = results
#         self.status = status

# class NeedleHaystack:   
#     def __init__(self, global_status, results, status):
#         self.global_status = global_status
#         self.results = results
#         self.status = status


import fire

from tromero.fine_tuning import TromeroModels, TromeroData, FineTuningJob, Datasets
import os 


class TromeroCli():
    def __init__(self):
        tromero_key=os.getenv("TROMERO_API_KEY")
        self.models = TromeroModels(tromero_key, raw_default=True)
        self.fine_tuning_jobs = FineTuningJob(tromero_key, raw_default=True)
        self.data = TromeroData(tromero_key)
        self.datasets = Datasets(tromero_key, raw_default=True)

def main():
    fire.Fire(TromeroCli)



import json
import re
class Message:
    def __init__(self, content, role="assistant"):
        self.content = content
        self.role = role

class Choice:
    def __init__(self, message):
        self.message = Message(message)

class Usage:
    def __init__(self, usage):
        self.completion_tokens = usage['completion_tokens']

class Response:
    def __init__(self, choices, usage=None):
        self.choices = choices
        self.usage = usage
        

def mock_openai_format(messages, usage):
    choices = [Choice(messages)]  # Create a list of Choice objects
    usage = Usage(usage)
    response = Response(choices, usage)
    return response

class StreamChoice:
    def __init__(self, message):
        self.delta = Message(message)

def mock_openai_format_stream(messages):
    choices = [StreamChoice(messages)]  # Create a list of Choice objects
    response = Response(choices)
    return response    

def tags_to_string(tags):
    return ','.join(tags)
        

def validate_file_content(file_path):
    # Check if the file extension is .jsonl
    if not file_path.endswith('.jsonl'):
        print("Error: File is not a .jsonl file.")
        return

    with open('sample_data_1000.jsonl', 'r') as file:
        content = file.read()

    lines = content.split('\n')
    index = 0
    has_error = False
    total_lines = 0

    def validate_line(index, total_lines):
        nonlocal has_error
        while index < len(lines) and not has_error:
            line = lines[index].strip()
            if line:
                try:
                    json_data = json.loads(line)
                    if 'messages' not in json_data or not isinstance(json_data['messages'], list):
                        raise ValueError(f'Invalid format on line {index + 1}: "messages" should be an array.')

                    has_user = False
                    has_assistant = False
                    roles = []
                    for message in json_data['messages']:
                        if 'role' not in message or 'content' not in message:
                            raise ValueError(f'Invalid format on line {index + 1}: Each message should have a "role" and "content".')
                        if message['role'] == 'user':
                            has_user = True
                        if message['role'] == 'assistant':
                            has_assistant = True
                        if message['role'] not in ['system', 'user', 'assistant']:
                            raise ValueError(f'Invalid role on line {index + 1}: Each message role should be either "system", "user", or "assistant".')
                        roles.append(message['role'])

                    if not has_user:
                        raise ValueError(f'Invalid format on line {index + 1}: Missing "user" message.')
                    if not has_assistant:
                        raise ValueError(f'Invalid format on line {index + 1}: Missing "assistant" message.')
                    
                    for i in range(1, len(roles)):
                        if roles[i] == roles[i - 1]:
                            raise ValueError(f'Invalid format on line {index + 1}: Roles should alternate starting with "user".')
                    if roles[0] not in ['user', 'system']:
                        raise ValueError(f'Invalid format on line {index + 1}: The first role should be "user" or "system".')
                    if roles[0] == 'system' and roles[1] != 'user':
                        raise ValueError(f'Invalid format on line {index + 1}: The role following "system" should be "user".')
                except ValueError as e:
                    print(str(e))
                    has_error = True
                    break
                except json.JSONDecodeError:
                    print(f"Error parsing JSON on line {index + 1}")
                    has_error = True
                    break
                total_lines += 1
            index += 1
        return total_lines

    total_lines = validate_line(index, total_lines)
    if not has_error:
        return True
    else:
        print("Validation encountered errors.")
        return False



from setuptools import setup, find_packages

setup(
    name="tromero",  # Replace with your package name
    version="0.0.4",  # Replace with your package's version
    author="Tromero",  # Replace with your name
    author_email="tadhg.amin@tromero.ai",  # Replace with your email address
    description="A short description of your package",  # Provide a short description
    long_description=open('README.md').read(),  # This will read your long description from README.md
    long_description_content_type='text/markdown',  # Indicates that the long description is in Markdown
    url="http://yourpackagehomepage.com",  # Replace with the URL to your package's homepage
    license="MIT",  # Replace with your chosen license
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=find_packages(),  # Automatically find all packages and subpackages
    include_package_data=True,
    install_requires=[
        "annotated-types==0.6.0",
        "anyio==4.3.0",
        "certifi==2024.7.4",
        "charset-normalizer==3.3.2",
        "distro==1.9.0",
        "h11==0.14.0",
        "httpcore==1.0.5",
        "httpx==0.27.0",
        "idna==3.7",
        "openai==1.16.1",
        "pydantic==2.6.4",
        "pydantic_core==2.16.3",
        "requests==2.32.2",
        "setuptools==70.0.0",
        "sniffio==1.3.1",
        "tqdm==4.66.3",
        "typing_extensions==4.10.0",
        "urllib3==2.2.2",
        "fire"
    ],
     entry_points={
        'console_scripts': [
            'tromero=tromero.cli:main'
        ]
    },
    python_requires='>=3.6',  # Specify the minimum version of Python required
)
